/*************************************************************************
 *                                                                       *
 * convert.c  - converts backup.ds into entry.nds, value.nds, block.nds, *
 *              and partitio.nds                                         *
 *                                                                       *
 * Programmer - Simple Nomad - Nomad Mobile Research Centre              *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/3/97     - Initial Revision                                         *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/5/97     - Completed program sans header handling                   *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/7/97     - Started header analysis                                  *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/9/97     - Rewrote all routines. Moved a lot of stuff out of main   *
 *              to clean up the program. Fixed a problem when trying to  *
 *              move past the header section.                            *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/10/97    - Fixed a real stupid pointer problem and a typo that      *
 *              altered the block record size. Tested successfully with  *
 *              small BACKUP.DS file.                                    *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/13/97    - Decided to simplify moving past the header and just look *
 *              for the first ENTRY.NDS record. Was able to delete a lot *
 *              of code as a result.                                     *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/28/97    - Switched the int used for record counting to a long to   *
 *              allow converting huge BACKUP.DS with tons of records.    *
 *              Added an endian conversion routine. This should be used  *
 *              if a copy of BACKUP.DS is retrieved from a little endian *
 *              server but convert is being compiled and run on a big    *
 *              endian box.                                              *
 *                                                                       *
 *-----------------------------------------------------------------------*
 * v.1.1                                                                 *
 *                                                                       *
 * 7/01/97    - Added -v switch to allow a visual clue the program has   *
 *              not hung.                                                *
 *                                                                       *
 *-----------------------------------------------------------------------*
 * v.2.0                                                                 *
 *                                                                       *
 * 2/10/98    - Added support for a couple of header files.              *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 3/10/98    - Minor code cleanup. Added -h switch for help screen.     *
 *                                                                       *  
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "globals.h"
#include "nds_stuf.h"

typedef struct just4bytes {
	uint32 offset;
} OFFSET;

#ifdef ENDIAN
union
{
  uint32 longData;
  uint8 shortData[4];
} output;

union
{
  uint32 longData;
  uint8 shortData[4];
} input;
#endif

/*
 * Global variables
 */
OFFSET inHeader;
FILE *fBackup;
FILE *fTemp;
uint32 SCAN;

/*
 * This routine switches byte ordering to get around the big 
 * endian -- little endian problem. I did this because systems
 * like AIX offer no solutions for a big endian reading a little
 * endian created file like BACKUP.DS. Send it a uint32 and it
 * returned the converted uint32.
 */
#ifdef ENDIAN
uint32 make_conversion(uint32 k) {
  int i;
  uint32 j;
  
  input.longData=k;
  for (i=0; i<4; i++)
    output.shortData[i]=input.shortData[3-i];
  return(output.longData);
}
#endif

/* 
 * This routine simply gets and returns the value of an offset.
 */
uint32 get_an_offset(void) {
  uint32 k;
  uint32 j;

  fseek(fBackup,SCAN,SEEK_SET);
  fread(&j,4,1,fBackup);
  k=j;
#ifdef ENDIAN
  k=make_conversion(k);
#endif
  return(k);
}

/*
 * This routine looks for 0xfffffffe. The first occurance of 
 * this will be in the first ENTRY.NDS record. Once found,
 * SCAN is updated and fBackup is seeked so that the next
 * offset to be read will be the number of records in 
 * ENTRY.NDS.
 */
int findFirstEntryRecord(void)
{
  uint32 k;
  int FOUND;
  long int counter;

  FOUND=FALSE;
  while(!(feof(fBackup)))
  {
    k=get_an_offset();
    if (k==0xfffffffe)
    {
      SCAN=(ftell(fBackup)-16);
      fseek(fBackup,SCAN,SEEK_SET);
      FOUND=TRUE;
      break;
    }
    SCAN++;
  }
  printf("\n");
  return(FOUND);
}

/*
 * This routine creates each file for NDS as it is called. It is
 * passed the number of records, the filename, and the file type.
 */
void makeNDS(long int records, char *filename, int filetype, int verbose) {
  ENTRY pentry;
  VALUE pvalue;
  BLOCK pblock;
  PARTITIO ppartitio;
  long int counter;
  int cc,cb,i;
  i=0;
  counter=0;

  fTemp = fopen(filename,"w+b");
  if (fTemp == NULL)
  {
    printf("\nUnable to open %s\n",filename);
    exit(1);
  }
  for(i=0;i<records;i++)
  {
    switch (filetype) {
    case typeEntry:
       cc=fread(&pentry,334,1,fBackup);
       cb=fwrite(&pentry,334,1,fTemp);
       break;
    case typeValue:
       cc=fread(&pvalue,64,1,fBackup);
       cb=fwrite(&pvalue,64,1,fTemp);
       break;
    case typeBlock:
       cc=fread(&pblock,128,1,fBackup);
       cb=fwrite(&pblock,128,1,fTemp);
       break;
    case typePartitio:
    default:
       cc=fread(&ppartitio,40,1,fBackup);
       cb=fwrite(&ppartitio,40,1,fTemp);
      break;
    }
    if (cc==0 || cb==0)
    {
      printf("\nProblems with record %ld\n",records);
      printf("Reading BACKUP.DS returned %d\n",cc);
      printf("Writing %s returned %d\nExiting...\n",filename,cb);
      exit(1);
    }
    if (verbose==TRUE)
    {
      counter++;
      printf("%08ld\b\b\b\b\b\b\b\b",counter);
    }
  }
  fclose(fTemp);
}

/*
 * This routine takes the size of a record as input, checks the
 * current offset, calculates the number of records, and advances
 * SCAN to point past the NDS file. It returns the number of
 * records found.
 */
long int countRecords(int size) {
  uint32 t;
  ldiv_t calc;
  long int j;

  t=get_an_offset();
  calc=ldiv(t,size);
  j = calc.quot;

  printf(" Scanning at : %08lx ** Offset value : %08lx ** Records found : %08ld\n",SCAN,t,j);
  SCAN += (calc.quot * size)+4;

  return(j);
}

void printHelp(int i)
{
  printf("USAGE: convert <options>\n\n");
  printf("  -h            this HELP screen\n");
  printf("  -v            VERBOSE mode\n\n");
  printf("EXAMPLES:\n");
  printf("  convert\n");
  printf("  convert -v\n");
  exit(i);
}
/*
 * Main program....
 */
void main(int argc, char *argv[])
{
  int i,verbose;
  long int j;
  uint32 k,l;
  ENTRY *pentry;
  VALUE *pvalue;
  BLOCK *pblock;
  PARTITIO *ppartition;
  ldiv_t calc;
  div_t calc2;

/* Say hello... */
  printf("CONVERT - extracts NDS files from BACKUP.DS\n");
  printf("Comments/bugs: pandora@nmrc.org\n");
  printf("http://www.nmrc.org/pandora\n");
  printf("1997 (c) Nomad Mobile Research Centre\n\n");

  SCAN=0;
  verbose=FALSE;

  fBackup=fopen("backup.ds","rb");
  if (fBackup == NULL)
  {
    printf("Unable to open backup.ds file.\n");
    exit(1);
  }

   /* process the command line */
   for (i=1 ; i<argc ; i++)
   {
      if (argv[i][0]=='-')
	 switch(argv[i][1])
	 {
	    case 'v':
	    case 'V':
	       verbose=TRUE;
	       break;
            case 'h':
            case 'H':
            case '?':
               printHelp(0);
               break;
	    default:
               printHelp(-1);
	       break;
	 }
   }
/* Find ENTRY.NDS portion of BACKUP.DS */
  i=findFirstEntryRecord();
  if(i==FALSE)
  {
    fclose(fBackup);
    printf("Unable to locate first ENTRY.NDS record.\n");
    exit(1);
  }
/* The next offset to be read in should be the number of
   ENTRY.NDS records */


/* Get ENTRY.NDS */
  printf("Searching for ENTRY.NDS\n");
  j=countRecords(334);
  calc=ldiv(j,4);
  if (calc.rem!=0) SCAN += 2;
  makeNDS(j,"ENTRY.NDS",typeEntry,verbose);
  printf("Built ENTRY.NDS           \n\n");

/* Get VALUE.NDS */
  printf("Searching for VALUE.NDS\n");
  j=countRecords(64);
  makeNDS(j,"VALUE.NDS",typeValue,verbose);
  printf("Built VALUE.NDS           \n\n");

/* Get BLOCK.NDS */
  printf("Searching for BLOCK.NDS\n");
  j=countRecords(128);
  makeNDS(j,"BLOCK.NDS",typeBlock,verbose);
  printf("Built BLOCK.NDS           \n\n");

/* Get PARTITIO.NDS */
  printf("Searching for PARTITIO.NDS\n");
  j=countRecords(40);
  makeNDS(j,"PARTITIO.NDS",typePartitio,verbose);
  printf("Built PARTITIO.NDS        \n");

  fclose(fBackup);
}


