/*************************************************************************
 *                                                                       *
 * crypto.h   - Jitsu-Disk's enhancements of itsme's proof-of-concept    *
 *              code. The Netware 3.x/4.x crypto algorithm.              *
 *                                                                       *
 *************************************************************************/

#ifndef CRYPTO_H
#define CRYPTO_H

#include "globals.h"

#ifdef __cplusplus
extern "C" {
#endif

const uint8 nyblTab[256]=  /* used by encrypt */
{/* 0   1   2   3   4   5   6   7   8   9   a   b   c   d   e   f  */
   0x7,0x8,0x0,0x8,0x6,0x4,0xE,0x4,0x5,0xC,0x1,0x7,0xB,0xF,0xA,0x8,
   0xF,0x8,0xC,0xC,0x9,0x4,0x1,0xE,0x4,0x6,0x2,0x4,0x0,0xA,0xB,0x9,
   0x2,0xF,0xB,0x1,0xD,0x2,0x1,0x9,0x5,0xE,0x7,0x0,0x0,0x2,0x6,0x6,
   0x0,0x7,0x3,0x8,0x2,0x9,0x3,0xF,0x7,0xF,0xC,0xF,0x6,0x4,0xA,0x0,
   0x2,0x3,0xA,0xB,0xD,0x8,0x3,0xA,0x1,0x7,0xC,0xF,0x1,0x8,0x9,0xD,
   0x9,0x1,0x9,0x4,0xE,0x4,0xC,0x5,0x5,0xC,0x8,0xB,0x2,0x3,0x9,0xE,
   0x7,0x7,0x6,0x9,0xE,0xF,0xC,0x8,0xD,0x1,0xA,0x6,0xE,0xD,0x0,0x7,
   0x7,0xA,0x0,0x1,0xF,0x5,0x4,0xB,0x7,0xB,0xE,0xC,0x9,0x5,0xD,0x1,
   0xB,0xD,0x1,0x3,0x5,0xD,0xE,0x6,0x3,0x0,0xB,0xB,0xF,0x3,0x6,0x4,
   0x9,0xD,0xA,0x3,0x1,0x4,0x9,0x4,0x8,0x3,0xB,0xE,0x5,0x0,0x5,0x2,
   0xC,0xB,0xD,0x5,0xD,0x5,0xD,0x2,0xD,0x9,0xA,0xC,0xA,0x0,0xB,0x3,
   0x5,0x3,0x6,0x9,0x5,0x1,0xE,0xE,0x0,0xE,0x8,0x2,0xD,0x2,0x2,0x0,
   0x4,0xF,0x8,0x5,0x9,0x6,0x8,0x6,0xB,0xA,0xB,0xF,0x0,0x7,0x2,0x8,
   0xC,0x7,0x3,0xA,0x1,0x4,0x2,0x5,0xF,0x7,0xA,0xC,0xE,0x5,0x9,0x3,
   0xE,0x7,0x1,0x2,0xE,0x1,0xF,0x4,0xA,0x6,0xC,0x6,0xF,0x4,0x3,0x0,
   0xC,0x0,0x3,0x6,0xF,0x8,0x7,0xB,0x2,0xD,0xC,0x6,0xA,0xA,0x8,0xD
};
/*
/ nyblTab reversed
/
/ 0   02 1c 2b 2c 30 3f 6e 72 89 9d ad b8 bf cc ef f1
/ 1   0a 16 23 26 48 4c 51 69 73 7f 82 94 b5 d4 e2 e5
/ 2   1a 20 25 2d 34 40 5c 9f a7 bb bd be ce d6 e3 f8
/ 3   32 36 41 46 5d 83 88 8d 93 99 af b1 d2 df ee f2
/ 4   05 07 15 18 1b 3d 53 55 76 8f 95 97 c0 d5 e7 ed
/ 5   08 28 57 58 75 7d 84 9c 9e a3 a5 b0 b4 c3 d7 dd
/ 6   04 19 2e 2f 3c 62 6b 87 8e b2 c5 c7 e9 eb f3 fb
/ 7   00 0b 2a 31 38 49 60 61 6f 70 78 cd d1 d9 e1 f6
/ 8   01 03 0f 11 33 45 4d 5a 67 98 ba c2 c6 cf f5 fe
/ 9   14 1f 27 35 4e 50 52 5e 63 7c 90 96 a9 b3 c4 de
/ a   0e 1d 3e 42 47 6a 71 92 aa ac c9 d3 da e8 fc fd
/ b   0c 1e 22 43 5b 77 79 80 8a 8b 9a a1 ae c8 ca f7
/ c   09 12 13 3a 4a 56 59 66 7b a0 ab d0 db ea f0 fa
/ d   24 44 4f 68 6d 7e 81 85 91 a2 a4 a6 a8 bc f9 ff
/ e   06 17 29 54 5f 64 6c 7a 86 9b b6 b7 b9 dc e0 e4
/ f   0d 10 21 37 39 3b 4b 65 74 8c c1 cb d8 e6 ec f4
*/

const uint8 crypTab[32]=   /* used by encrypt & encryptp */
{

0x48,0x93,0x46,0x67,0x98,0x3D,0xE6,0x8D,0xB7,0x10,0x7A,0x26,0x5A,0xB9,0xB1,0x35,

0x6B,0x0F,0xD5,0x70,0xAE,0xFB,0xAD,0x11,0xF4,0x47,0xDC,0xA7,0xEC,0xCF,0x50,0xC0
};

/* The characters used for brute force cracking */
char data[68]={"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789,./<>?;':\"[]{}`~!@#$%^&*()_-+=|"};

/* 
 * Global variables
 */
uint8 crypt2data[96];
uint8 crypt2_c_val;

/*
 * START OF ITSME ROUTINES modified by Jitsu-Disk feb/98
 */

/*
 * calcEbis called by encryptp
 */
uint8 calcEbis(int index)
{int k,i;

 i=index&0x1f;
 k=(crypt2_c_val+i)&0x1f;
 
 if (k<i) 
  return(crypt2data[index]=
   (crypt2data[index-i+k]-crypTab[i])^(crypt2data[index-32]+crypt2_c_val));

 return(crypt2data[index]=
  (crypt2data[index-32-i+k]-crypTab[i])^(crypt2data[index-32]+crypt2_c_val));
}

/*
 * XOR password with object ID, used for initialize in Main
 */
void xorwithid(uint32 id, uint32 *buf)
{int i;
 for (i=0 ; i<8 ; i++) *buf++ ^= id;
}

/*
 * Prepare the password by lengthening... Used for init in Main
 */
unsigned short preparepw(uint8 *pw, int pwLen, uint8 *dst)
{uint8 *p;
 unsigned short turns=1;
 int i=0;

 p=pw;
 do {if (pw+pwLen==p) {p=pw;
                       *dst++=crypTab[i];
                       turns++;}
     else *dst++=*p++;
     i++;
    } while (i<32);

 return turns;
}

/* 
 * encryptp() has changed quite a bit. Comments in this routine
 * are Jitsu-Disk's - SN
 */
int encryptp() /*parameters became useless*/
{int i; 

/* This was not an easy one, but the double-recursive calc_c,calcElement
   is now broken down to a linear iterative formula */

 crypt2data[32]=(crypt2data[0]-crypTab[0])^(crypt2data[0]);
 crypt2_c_val=crypt2data[32];
/* Wow redundancy DOES bring improvement...*/
 for(i=1;i<32;i++)
 {
  crypt2data[i+32]=

(crypt2data[(crypt2_c_val&0x1f)+i]-crypTab[i])^(crypt2data[i]+crypt2_c_val);

  crypt2_c_val=crypt2_c_val+crypt2data[i+32];
 }

 crypt2data[64]=

(crypt2data[32+(crypt2_c_val&0x1f)]-crypTab[0])^(crypt2data[32]+crypt2_c_val);

   /* Heuristic : Calling calcElement is very CPU consuming,
      so let's try to compare the 4 first bits of the first
      nibble first. Proves to be a good solution 93% of the time */
 if (nyblTab[crypt2data[64]]^pPassword.hash[0]&15) return(FALSE);
 crypt2_c_val=crypt2_c_val+crypt2data[64];

 /* Heuristic : In 90% of bad match, nibble 0 or 1 rejected */
   crypt2data[65]=

(crypt2data[(crypt2_c_val&0x1f)+33]-crypTab[1])^(crypt2data[33]+crypt2_c_val);

 if ( (nyblTab[crypt2data[64]]|nyblTab[crypt2data[65]]<<4)
       !=pPassword.hash[0]) return(FALSE);
 crypt2_c_val=crypt2_c_val+crypt2data[65];

   crypt2data[66]=

(crypt2data[(crypt2_c_val&0x1f)+34]-crypTab[2])^(crypt2data[34]+crypt2_c_val);

   crypt2_c_val=crypt2_c_val+calcEbis(66);
   crypt2data[67]=

(crypt2data[(crypt2_c_val&0x1f)+35]-crypTab[3])^(crypt2data[35]+crypt2_c_val);

   if ( (nyblTab[crypt2data[66]]|nyblTab[crypt2data[67]]<<4)
       !=pPassword.hash[1]) return(FALSE);
   crypt2_c_val=crypt2_c_val+crypt2data[67];

   /*Crypt2 & shrinkbuf in short*/
   for (i=2 ; i<16 ; i++)
    {crypt2_c_val=crypt2_c_val+calcEbis(64+i+i);
     crypt2_c_val=crypt2_c_val+calcEbis(64+i+i+1);
     if ( (nyblTab[crypt2data[64+i+i]]|
           nyblTab[crypt2data[64+i+i+1]]<<4)
         !=pPassword.hash[i]) return(FALSE);
    }

   return(TRUE);
} 

#ifdef __cplusplus
}
#endif

#endif /* CRYPTO_H */
